from django.conf import settings
from rest_framework.settings import APISettings


FIREBASE_AUTH_SETTINGS = "FIREBASE_AUTH"

USER_SETTINGS = getattr(settings, FIREBASE_AUTH_SETTINGS)

DEFAULTS = {"SERVICE_ACCOUNT_KEY_FILE": "", "EMAIL_VERIFICATION": False}

IMPORT_STRINGS = ()

firebase_auth_settings = APISettings(USER_SETTINGS, DEFAULTS, IMPORT_STRINGS)



from django.contrib.auth import get_user_model
from django.utils.translation import gettext as _
from rest_framework import exceptions
from rest_framework.authentication import BaseAuthentication, get_authorization_header

from firebase_admin import auth, credentials, initialize_app

from firebase_auth.settings import firebase_auth_settings


User = get_user_model()

cred = credentials.Certificate(firebase_auth_settings.SERVICE_ACCOUNT_KEY_FILE)
initialize_app(cred)


class BaseFirebaseAuthentication(BaseAuthentication):
    """
    Base implementation of token based authentication using firebase.
    """

    www_authenticate_realm = "api"
    auth_header_prefix = "Bearer"
    uid_field = User.USERNAME_FIELD

    def authenticate(self, request):
        """
        Returns a two-tuple of `User` and decoded firebase payload if a valid signature
        has been supplied. Otherwise returns `None`.
        """
        firebase_token = self.get_token(request)

        if not firebase_token:
            return None

        try:
            payload = auth.verify_id_token(firebase_token)
        except ValueError:
            msg = _("Invalid firebase ID token.")
            raise exceptions.AuthenticationFailed(msg)
        except (
            auth.ExpiredIdTokenError,
            auth.InvalidIdTokenError,
            auth.RevokedIdTokenError,
        ):
            msg = _("Could not log in.")
            raise exceptions.AuthenticationFailed(msg)

        user = self.authenticate_credentials(payload)

        return (user, payload)

    def get_token(self, request):
        """
        Returns the firebase ID token from request.
        """
        auth = get_authorization_header(request).split()

        if not auth or auth[0].lower() != self.auth_header_prefix.lower().encode():
            return None

        if len(auth) == 1:
            msg = _("Invalid Authorization header. No credentials provided.")
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = _(
                "Invalid Authorization header. Token string should not contain spaces."
            )
            raise exceptions.AuthenticationFailed(msg)

        return auth[1]

    def authenticate_credentials(self, payload):
        """
        Returns an user that matches the payload's user uid and email.
        """
        if payload["firebase"]["sign_in_provider"] == "anonymous":
            msg = _("Firebase anonymous sign-in is not supported.")
            raise exceptions.AuthenticationFailed(msg)

        if firebase_auth_settings.EMAIL_VERIFICATION:
            if not payload["email_verified"]:
                msg = _("User email not yet confirmed.")
                raise exceptions.AuthenticationFailed(msg)

        uid = payload["uid"]

        try:
            user = self.get_user(uid)
        except User.DoesNotExist:
            firebase_user = auth.get_user(uid)
            user = self.create_user_from_firebase(uid, firebase_user)

        return user

    def get_user(self, uid: str) -> User:
        """Returns the user with given uid"""
        raise NotImplementedError(".get_user() must be overriden.")

    def create_user_from_firebase(
        self, uid: str, firebase_user: auth.UserRecord
    ) -> User:
        """Creates a new user with firebase info"""
        raise NotImplementedError(".create_user_from_firebase() must be overriden.")

    def authenticate_header(self, request):
        return '{} realm="{}"'.format(
            self.auth_header_prefix, self.www_authenticate_realm
        )


class FirebaseAuthentication(BaseFirebaseAuthentication):
    """
    Token based authentication using firebase.

    Clients should authenticate by passing a Firebase ID token in the
    Authorizaiton header using Bearer scheme.
    """

    def get_user(self, uid: str) -> User:
        return User.objects.get(**{self.uid_field: uid})

    def create_user_from_firebase(
        self, uid: str, firebase_user: auth.UserRecord
    ) -> User:
        fields = {self.uid_field: uid, "email": firebase_user.email}

        return User.objects.create(**fields)



import os
import re
import shutil
import sys

from setuptools import setup


def get_version(package):
    """
    Return package version as listed in `__version__` in `init.py`.
    """
    with open(os.path.join(package, "__init__.py"), "rb") as init_py:
        src = init_py.read().decode("utf-8")
        return re.search("__version__ = ['\"]([^'\"]+)['\"]", src).group(1)


name = "django-rest-firebase-auth"
version = get_version("firebase_auth")
package = "firebase_auth"
description = "Firebase authentication for Django REST framework"
url = "https://github.com/walison17/django-rest-firebase-auth"
author = "Walison Filipe"
author_email = "walisonfilipe@hotmail.com"
license = "MIT"
install_requires = [
    "firebase-admin>=2.0.0"
]


def read(*paths):
    """
    Build a file path from paths and return the contents.
    """
    with open(os.path.join(*paths), "r") as f:
        return f.read()


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [
        dirpath
        for dirpath, dirnames, filenames in os.walk(package)
        if os.path.exists(os.path.join(dirpath, "__init__.py"))
    ]


def get_package_data(package):
    """
    Return all files under the root package, that are not in a
    package themselves.
    """
    walk = [
        (dirpath.replace(package + os.sep, "", 1), filenames)
        for dirpath, dirnames, filenames in os.walk(package)
        if not os.path.exists(os.path.join(dirpath, "__init__.py"))
    ]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename) for filename in filenames])
    return {package: filepaths}


if sys.argv[-1] == "publish":
    if os.system("pip freeze | grep twine"):
        print("twine not installed.\nUse `pip install twine`.\nExiting.")
        sys.exit()

    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")

    shutil.rmtree("dist")
    shutil.rmtree("build")
    shutil.rmtree("django_rest_firebase_auth.egg-info")

    print("You probably want to also tag the version now:")
    print("  git tag -a {0} -m 'version {0}'".format(version))
    print("  git push --tags")

    sys.exit()


setup(
    name=name,
    version=version,
    url=url,
    license=license,
    description=description,
    long_description_content_type="text/markdown",
    long_description=read("README.md"),
    author=author,
    author_email=author_email,
    packages=get_packages(package),
    package_data=get_package_data(package),
    install_requires=install_requires,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Environment :: Web Environment",
        "Framework :: Django",
        "Framework :: Django :: 2.2",
        "Framework :: Django :: 3.0",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Internet :: WWW/HTTP",
    ],
)
